/*
 * Billboard.java
 *
 * Created: April 7, 2009, 10:18 PM
 *
 */
package irrtum.engine;

import irrtum.game.Camera;
import org.lwjgl.opengl.*;

/**
 * An object that wraps around an Image, but renders itself in the game
 * world as a flat image oriented toward the camera.
 *
 * @author Stephen Whitmore
 *
 */
public class Billboard
{
    public static void draw(Texture tex, Camera cam, Vector3 loc, float scale, Vector2 frameSize, int frame, Vector3 col)
    {
        // Frame-locating/drawing values
        float frw = frameSize.x / (float)tex.getWidth();
        float frh = frameSize.y / (float)tex.getHeight();
        int frames = tex.getWidth() / (int)frameSize.x;
        int framesY = tex.getHeight() / (int)frameSize.y;
        frames *= framesY;
        float frw_steps = frw * (frame % frames);
        float frh_steps = frh * (frame / frames);
        
        // Offsets for centre/scaling/etc.
        float cox = 0.5f;
        float coy = 0.5f;
        float rs = 0.5f;
        float bs = 0.5f;
        float wr = tex.getRealWidth() / (float)tex.getWidth();
        float wh = tex.getRealHeight() / (float)tex.getHeight();
        
        // Preparing OpenGL states for image drawing
        GL11.glPushMatrix();
        GL11.glLoadIdentity();
        cam.doTransform();
        GL11.glTranslatef(loc.x, loc.y, loc.z);
        GL11.glScalef(scale,scale,scale);
        float rot = (float)Math.toDegrees( Math.atan2(cam.pos.x-loc.x, cam.pos.y-loc.y) );
        GL11.glRotatef(rot, 0,0,1);
        tex.bind();
        
        // Draw the billboard
        GL11.glColor3f(col.x, col.y, col.z);
        GL11.glBegin(GL11.GL_QUADS);
            GL11.glTexCoord2f(frw_steps,frh_steps);
            GL11.glVertex2f(-cox,-coy);
            
            GL11.glTexCoord2f(frw_steps+frw,frh_steps);
            GL11.glVertex2f(rs,-coy);

            GL11.glTexCoord2f(frw_steps+frw,frh_steps+frh);
            GL11.glVertex2f(rs,bs);

            GL11.glTexCoord2f(frw_steps,frh_steps+frh);
            GL11.glVertex2f(-cox,bs);
        GL11.glEnd();
        
        // Clean up changed state information
        GL11.glPopMatrix();
        GL11.glColor4f(1,1,1,1);
    }
}
