/*
 * Input.java
 *
 * Created on September 22, 2007, 9:21 PM
 *
 */
package irrtum.engine;

import org.lwjgl.input.*;
import org.lwjgl.opengl.*;

/**
 * Wrapper over the LWJGL keyboard and mouse routines, to simplify and extend user input.
 *
 * @author Stephen Whitmore
 */
public class Input
{
    public static final int MOUSE_LEFT = 256;
    public static final int MOUSE_RIGHT = 257;
    public static final int MOUSE_MIDDLE = 258;
    
    private static int[] keys;
    private static int[] buttons;
    private static Vector2 mouse, mouseMickey;
    
    static
    {
        mouse = new Vector2();
        mouseMickey = new Vector2();
        keys = new int[259];
        buttons = new int[3];
    }
    
    public static void update()
    {
        for(int i=0; i < 256; i++)
        {
            if(Keyboard.isKeyDown(i))
                keys[i]++;
            else
            {
                if(keys[i] > 1)
                    keys[i] = -1;
                else
                    keys[i] = 0;
            }
        }
        
        for(int i=0; i < buttons.length; i++)
        {
            if(Mouse.isButtonDown(i))
            {
                buttons[i]++;
                keys[256+i]++;
            }
            else
            {
                if(buttons[i] > 1)
                {
                    buttons[i] = -1;
                    keys[256+i] = -1;
                }
                else
                {
                    buttons[i] = 0;
                    keys[256+i] = 0;
                }
            }
        }

        Vector2 newMouse = new Vector2( Mouse.getX(), Display.getDisplayMode().getHeight()-Mouse.getY() );
        mouseMickey = newMouse.sub(mouse);
        mouse = newMouse;
    }
    
    public static boolean isKeyHit(int k)
    {
        return keys[k] == 1;
    }
    
    public static boolean isKeyDown(int k)
    {
        return keys[k] > 0;
    }
    
    public static boolean isKeyReleased(int k)
    {
        return keys[k] == -1;
    }
    
    public static boolean isButtonHit(int b)
    {
        return buttons[b] == 1;
    }
    
    public static boolean isButtonDown(int b)
    {
        return buttons[b] > 0;
    }
    
    public static boolean isButtonReleased(int b)
    {
        return buttons[b] == -1;
    }
    
    public static int getMouseX()
    {
        return (int)mouse.getX();
    }
    
    public static int getMouseY()
    {
        return (int)mouse.getY();
    }
    
    public static Vector2 getMousePos()
    {
        return Vector2.create(mouse);
    }

    public static Vector2 getMouseMickey()
    {
        return Vector2.create(mouseMickey);
    }
    
    public static String getKeyName(int key)
    {
        if(key == 256) return "MOUSE-LEFT";
        else if(key == 257) return "MOUSE-RIGHT";
        else if(key == 258) return "MOUSE-MIDDLE";
        else return Keyboard.getKeyName(key);
    }
    
    public static int getKeyIndex(String key)
    {
        if(key.equals("MOUSE-LEFT")) return 256;
        else if(key.equals("MOUSE-RIGHT")) return 257;
        else if(key.equals("MOUSE-MIDDLE")) return 258;
        else return Keyboard.getKeyIndex(key);
    }
}
