/*
 * Log.java
 *
 * Created: 07-Apr-2009, 1:04 PM
 * Author: Stephen Whitmore
 *
 */
package irrtum.engine;

import java.util.*;
import java.io.*;
import java.text.SimpleDateFormat;
import java.awt.Color;


/**
 * Static class for logging information and events that occur during a client-side or
 * server-side session of the game. Information may be logged on several different channels
 * for maximized tailoring of the data presented.
 *
 */
public class Log
{
	static LinkedList<LogMessage> msgs;
	
    static
    {
        msgs = new LinkedList<LogMessage>();
    }
    
    public static void write(String txt, LogChannel chan)
    {
        LogMessage msg = new LogMessage();
        msg.text = txt;
        msg.type = chan;
        msg.time = new Date();
        msgs.add(msg);
        System.out.println(chan.toString()+": "+txt);
    }
    
    public static void save(boolean sys)
    {
        return;
        /*
        ArrayList<LogMessage> log = sys ? syslog : chatlog;
        String sep = File.separator;
        
	// Generate unique filenamename based on date/time/server
        String fname="log.html";
        Date now = new Date();
        SimpleDateFormat date = new SimpleDateFormat("MMMMM dd yyyy");
        try
        {
            fname = date.format(now);
        }
        catch(Exception e) { }
        
        // Create directory path to log file
        String basePath = System.getProperty("user.home")+sep+Config.baseDir+sep;
        String path = basePath;
        makeDir(basePath+"logs");
        path += "logs"+sep;
        if(sys)
        {
            makeDir(basePath+"logs"+sep+"syslogs");
            path += "syslogs"+sep;
        }
        else
        {
            makeDir(basePath+"logs"+sep+"chatlogs");
            path += "chatlogs"+sep;
        }
        
        date = new SimpleDateFormat("yyyy");
        makeDir(path+date.format(now));
        path += date.format(now)+sep;
        
        date = new SimpleDateFormat("MMMMM");
        makeDir(path+date.format(now));
        path += date.format(now)+sep;        
        
        date = new SimpleDateFormat("dd");
        makeDir(path+date.format(now));
        path += date.format(now)+sep;
        fname = path + fname;
        
        // Add parenthesized number to end of filename if file exists
        boolean found = !(new File(fname+".html")).exists();
        int num = 1;
        while(!found)
        {
            String new_name = fname + " ("+num+").html";
            found = !(new File(new_name)).exists();
            if(found)
                fname = fname + " ("+num+")";
            num++;
        }        
	
	// Write logged data to file
        try
	{
	    BufferedWriter bw = new BufferedWriter( new FileWriter( new File(fname+".html") ) );
	    bw.write("<html><body bgcolor=#000000><font face=\"fixedsys\">");
	    
	    for(int i=0; i < log.size(); i++)
	    {
		LogMessage line = log.get(i);
		String msg = line.text;
		LogChannel type = line.type;
                Date time = line.time;
		
                String col_1 = type.getColour();
                String col_2 = Log.colourToHexString( Color.decode(col_1).darker() );
                String out="";
                
                if(useTimeStamping)
                {
                    SimpleDateFormat format = new SimpleDateFormat("(hh:mm:ss a)");
                    out = "<font face=\"arial\" size=1 color="+col_2+">"+format.format(time)+"</font> ";
                }
                
                out += "<font color="+col_1+">["+type.toString()+"]</font> <font color="+col_2+">"+msg+"</font><br>";
                bw.write(out);
	    }
	    bw.write("</font></body></html>");
	    
	    bw.close();
	}
	catch(Exception e)
	{
	    System.out.println("Could not save log file: "+e);
	}
         */
    }
        
    private static void makeDir(String dir)
    {
        File file = new File(dir);
        file.mkdir();
    }

    public static String colourToHexString(Color c)
    {
        String r = Integer.toHexString( c.getRed() );
        if(r.length() == 1) r = "0" + r;
        String g = Integer.toHexString( c.getGreen() );
        if(g.length() == 1) g = "0" + g;
        String b = Integer.toHexString( c.getBlue() );
        if(b.length() == 1) b = "0" + b;
        return "#" + r + g + b;
    }

    public static SimpleDateFormat getCurrentTimeStamp()
    {
        return new SimpleDateFormat("(hh:mm:ss a)");
    }
}

class LogMessage
{
	public String text;
	public LogChannel type;
	public Date time;
}