/*
 * ParticleEngine.java
 *
 * Created on December 9, 2007, 10:31 AM
 *
 */
package irrtum.engine;

import irrtum.game.Entity;
import java.util.*;
import java.awt.Color;
import org.lwjgl.opengl.*;

/**
 * A static class responsible for spawning, updating, drawing, and managing particles in the game world.
 *
 * @author Stephen Whitmore
 */
public class ParticleEngine
{
    static LinkedList<Particle> particles;
    static LinkedList<Particle> dead;
    
    static
    {
        particles = new LinkedList<Particle>();
        dead = new LinkedList<Particle>();
    }
    
    private static void add(Particle p)
    {
        particles.add(p);
    }
    
    private static void remove(Particle p)
    {
        particles.remove(p);
    }
    
    public static void render()
    {
        for(Particle p : particles)
        {
            p.render();
        }
    }

    public static void update()
    {
        dead.clear();
        for(Particle p : particles)
        {
            p.update();
            if(p.isDead())
                dead.add(p);
        }
        for(Particle p : dead)
        {
            particles.remove(p);
        }
    }
/*
    public static void spawnParticles(int num, String imgName, float x, float y, int z, int randRadius, Color scol,
                                         Color ecol, float scStart, float scEnd, float minAng, float maxAng, float minSpeed,
                                         float maxSpeed, float friction, int lifetime, int blendMode)
    {
        Random rand = new Random();
        
        // TODO: Smarter culling that takes the targetPos of the camera into account
        // Perform culling
        //Rectangle scr = Camera.getScreenRect();
        //scr.grow(randRadius*2, randRadius*2);
        //if( !scr.contains(x,y) )
          //  return;
        
        Image img = ResourceManager.getImage(imgName);
        if(img == null)
            return;
        
        // Spawn particles
        for(int i=0; i < num; i++)
        {
            Particle p = new Particle();
            p.setZ(z);
            p.setImage(img);
            float offang = (float)Math.toRadians( rand.nextInt(360) );
            float offamt = rand.nextFloat() * randRadius;
            p.setPos( new Vector2( x + Math.cos(offang) * offamt - img.getFrameWidth()/2, y + Math.sin(offang) * offamt - img.getFrameHeight()/2 ) );
            p.setColours(scol, ecol);
            p.setRot( rand.nextFloat() * (maxAng-minAng) + minAng );
            float ang = (float)Math.toRadians( p.getRot() );
            float spd = rand.nextFloat() * (maxSpeed-minSpeed) + minSpeed;
            p.setVelocity( new Vector2(Math.cos(ang) * spd, Math.sin(ang) * spd) );
            p.setCentre(new Vector2(0.5f,0.5f));
            p.setFriction(friction);
            p.setLifetime(lifetime);
            p.setBlend(blendMode);
            p.setScaling(scStart, scEnd);
            add(p);
        }
    }
*/
/*
    public static void spawnProjectileProps(int num, String propName, float x, float y, float z, int randRadius, 
                                         float minAng, float maxAng, float minSpeed, float maxSpeed,
                                         float minZSpeed, float maxZSpeed, int lifetime, Vector3 force, Player owner)
    {
        Random rand = new Random();
        
        // Spawn props
        for(int i=0; i < num; i++)
        {
            ProjectileProp p = (ProjectileProp)Prop.createProp(propName);
            p.setOwner(owner);
            p.setLifetime(lifetime);
            float offang = (float)Math.toRadians( rand.nextInt(360) );
            float offamt = rand.nextFloat() * randRadius;
            p.setPos( new Vector3( x + (float)Math.cos(offang) * offamt - p.getWidth()*p.getCentre().x,
                                   y + (float)Math.sin(offang) * offamt - p.getHeight()*p.getCentre().y,
                                   z) );
            p.setRot( rand.nextFloat() * 360.0f );
            float ang = rand.nextFloat() * (maxAng-minAng) + minAng;
            ang = (float)Math.toRadians( ang );
            float spd = rand.nextFloat() * (maxSpeed-minSpeed) + minSpeed;
            float zspd = rand.nextFloat() * (maxZSpeed-minZSpeed) + minZSpeed;
            p.applyForce( new Vector3((float)Math.cos(ang) * spd, (float)Math.sin(ang) * spd, zspd) );
            if(force != null)
                p.applyForce( force );
        }
    }
    
    public static void spawnProps(int num, String propName, float x, float y, float z, int randRadius, 
                                         float minAng, float maxAng, float minSpeed, float maxSpeed,
                                         float minZSpeed, float maxZSpeed, int lifetime, Vector3 force)
    {
        Random rand = new Random();
        
        // Perform culling
        Rectangle scr = Camera.getScreenRect();
        scr.grow(randRadius*4, randRadius*4);
        if( !scr.contains(x,y) )
            return;
        
        // Spawn props
        for(int i=0; i < num; i++)
        {
            Prop p = Prop.createProp(propName);
            p.setLifetime(lifetime);
            float offang = (float)Math.toRadians( rand.nextInt(360) );
            float offamt = rand.nextFloat() * randRadius;
            p.setPos( new Vector3( x + (float)Math.cos(offang) * offamt - p.getImage().getFrameWidth()/2,
                                   y + (float)Math.sin(offang) * offamt - p.getImage().getFrameHeight()/2,
                                   z) );
            p.setRot( rand.nextFloat() * 360.0f );
            float ang = rand.nextFloat() * (maxAng-minAng) + minAng;
            ang = (float)Math.toRadians( ang );
            float spd = rand.nextFloat() * (maxSpeed-minSpeed) + minSpeed;
            float zspd = rand.nextFloat() * (maxZSpeed-minZSpeed) + minZSpeed;
            p.applyForce( new Vector3((float)Math.cos(ang) * spd, (float)Math.sin(ang) * spd, zspd) );
            if(force != null)
                p.applyForce( force );
        }
    }
*/
}

class Particle extends Entity
{
    Vector3 vel;
    Entity parent;
    float friction, scaleStart, scaleEnd;
    int lifetime, maxLifetime;
    Color startCol, endCol;
    
    Particle()
    {
        super();
        vel = new Vector3();
    }
    
    public void update()
    {
        vel.set( vel.scale(friction) );
        pos.set(pos.add(vel));
        lifetime--;
        
        if(lifetime <= 0)
            kill();
    }
    
    public void kill()
    {
        this.setDead(true);
    }
    
    public void setVelocity(Vector3 v)
    {
        vel.set(v);
    }
    public void setParent(Entity par)
    {
        parent = par;
    }
    public void setFriction(float fric)
    {
        friction = fric;
    }
    public void setLifetime(int life)
    {
        lifetime = maxLifetime = life;
    }
    public void setColours(Color s, Color e)
    {
        startCol = s;
        endCol = e;
    }
    public void setScaling(float ss, float se)
    {
        scaleStart = ss;
        scaleEnd = se;
    }
    
    public void render()
    {
        GL11.glDisable(GL11.GL_DEPTH_TEST);
        int r,g,b,a;
        float per = lifetime / (float)maxLifetime;
        Color col = new Color( (int)(startCol.getRed() * per + endCol.getRed() * (1-per)),
                               (int)(startCol.getGreen() * per + endCol.getGreen() * (1-per)),
                               (int)(startCol.getBlue() * per + endCol.getBlue() * (1-per)),
                               (int)(startCol.getAlpha() * per + endCol.getAlpha() * (1-per)) );
        float sc = scaleStart * per + scaleEnd * (1-per);
        scale = sc;
//        if(texture != null)
//            texture.draw(pos,centre,rot,scale,col,col,col,col,blend,-1,frame,world);
        GL11.glEnable(GL11.GL_DEPTH_TEST);
    }
}


