/*
 * Shapes.java
 *
 * Created on September 21, 2007, 8:07 PM
 *
 */
package irrtum.engine;

import org.lwjgl.opengl.*;
import java.awt.Color;

/**
 * A collection of renderable 2D primitives using OpenGL 3D primitives.
 *
 * @author Stephen Whitmore
 */
public class Shapes
{
    public static float Z;
    static boolean world;
    static boolean enableTextures;
    
    static
    {
        Z = 200;
    }
    
    public static void pushToScreenCoords()
    {
        GL11.glPushMatrix();
        GL11.glLoadIdentity();
    }
    
    public static void popCoords()
    {
        GL11.glPopMatrix();
    }
    
    public static void preDraw()
    {
        enableTextures = GL11.glIsEnabled(GL11.GL_TEXTURE_2D);
        GL11.glDisable(GL11.GL_TEXTURE_2D);
        if(!world) { //GL11.glDepthMask(false);
        //GL11.glDisable(GL11.GL_DEPTH_TEST);
        //GL11.glDepthMask(false);
        GL11.glDepthFunc(GL11.GL_ALWAYS); }
    }
    
    public static void postDraw()
    {
        setColour(Color.WHITE);
        if(enableTextures)
            GL11.glEnable(GL11.GL_TEXTURE_2D);
        //GL11.glEnable(GL11.GL_DEPTH_TEST);
        GL11.glDepthMask(true);
        GL11.glDepthFunc(GL11.GL_LESS);
    }
    
    public static void drawOverAll()
    {
        world = false;
        Z = 200;
    }
    
    public static void drawInWorld(float z)
    {
        Z = z;
        world = true;
    }
    
    public static void setColour(Color c1)
    {
        GL11.glColor4ub( (byte)c1.getRed(), (byte)c1.getGreen(), (byte)c1.getBlue(), (byte)c1.getAlpha() );
    }
    public static void setColour(Vector3 c)
    {
        GL11.glColor3f(c.x, c.y, c.z);
    }
    
    public static void setLineWidth(float lw)
    {
        GL11.glLineWidth(lw);
    }
    
    public static void createVertex(Vector2 v)
    {
        GL11.glVertex3f(v.x, v.y, Z);
    }
    public static void createVertex(Vector3 v)
    {
        GL11.glVertex3f(v.x, v.y, v.z);
    }
    
    public static void drawLine(Vector2 start, Vector2 end, Color c1, Color c2)
    {
        preDraw();
        
        GL11.glBegin(GL11.GL_LINES);
            setColour(c1);
            GL11.glVertex3f(start.getX(), start.getY(), Z);
            setColour(c2);
            GL11.glVertex3f(end.getX(), end.getY(), Z);
        GL11.glEnd();
        
        postDraw();
    }
    
    public static void drawCircle(Vector2 centre, float radius, Color c1)
    {
        preDraw();
        setColour(c1);
        
        int step = (int)(radius / 2);
        
        GL11.glBegin(GL11.GL_LINE_STRIP);
            for(int i=0; i < 360+step; i += step)
            {
                float dx = (float)Math.cos( Math.toRadians(i) ) * radius;
                float dy = (float)Math.sin( Math.toRadians(i) ) * radius;
                GL11.glVertex3f( centre.getX() + dx, centre.getY() + dy, Z );
            }
        GL11.glEnd();
        
        postDraw();
    }
    
    public static void fillCircle(Vector2 centre, float radius, Color c1)
    {
        preDraw();
        setColour(c1);
        
        int step = (int)(radius / 2);
        
        GL11.glBegin(GL11.GL_TRIANGLE_FAN);
            GL11.glVertex3f( centre.getX(), centre.getY(), Z );
            for(int i=0; i < 360+step; i += step)
            {
                float dx = (float)Math.cos( Math.toRadians(i) ) * radius;
                float dy = (float)Math.sin( Math.toRadians(i) ) * radius;
                GL11.glVertex3f( centre.getX() + dx, centre.getY() + dy, Z );
            }
        GL11.glEnd();
        
        postDraw();
    }
    
    public static void fillRect(float x, float y, float w, float h, Color c1, Color c2, Color c3, Color c4)
    {
        Vector2 u, v;
        u = Vector2.create(x,y);
        v = Vector2.create(w,h);
        fillRect(u, v, c1,c2,c3,c4);
    }
    public static void fillRect(Vector2 topleft, Vector2 size, Color c1, Color c2, Color c3, Color c4)
    {
        preDraw();
        Vector2 v = Vector2.create();
        
        GL11.glBegin(GL11.GL_QUADS);
            setColour(c1);
            createVertex(topleft);
            
            setColour(c2);
            v.set(topleft.x+size.x, topleft.y);
            createVertex(v);
            
            setColour(c3);
            v.set(topleft.x+size.x, topleft.y+size.y);
            createVertex(v);
            
            setColour(c4);
            v.set(topleft.x, topleft.y+size.y);
            createVertex(v);
        GL11.glEnd();
        
        postDraw();
    }

    public static void drawRect(float x, float y, float w, float h, Color c)
    {
        Vector2 u, v;
        u = Vector2.create(x,y);
        v = Vector2.create(w,h);
        drawRect(u, v, c);
    }
    public static void drawRect(Vector2 topleft, Vector2 size, Color c1)
    {
        preDraw();
        Vector2 v = Vector2.create();
        
        setColour(c1);
        GL11.glBegin(GL11.GL_LINE_STRIP);
            createVertex(topleft);
            
            v.set(topleft.x+size.x, topleft.y);
            createVertex(v);
            
            v.set(topleft.x+size.x, topleft.y+size.y);
            createVertex(v);
            
            v.set(topleft.x, topleft.y+size.y);
            createVertex(v);
            
            v.set(topleft.x, topleft.y);
            createVertex(v);
        GL11.glEnd();
        
        postDraw();
    }
    
    public static void fillTriangle(Vector2 a, Vector2 b, Vector2 c, Color col)
    {
        preDraw();
        
        setColour(col);
        GL11.glBegin(GL11.GL_TRIANGLES);
            createVertex(a);
            createVertex(b);
            createVertex(c);
        GL11.glEnd();
        
        postDraw();
    }
}
