/*
 * Vector2.java
 *
 * Created: July 2, 2007, 4:44 PM
 * Author: Stephen Whitmore
 *
 */
package irrtum.engine;

/**
 * Utility class with the purpose of handling 2-dimensional coordinates
 * with relative ease.
 */
public class Vector2
{
    public static final float EPSILON = 0.00001f;
    private static final int POOL_SIZE = 100000;
    public float x, y;
    boolean inPool;
    public static int poolPos = 0, maxPoolPos = 0;
    public static Vector2[] pool = new Vector2[POOL_SIZE];
    public static long vecs=0;
    
    public boolean isPooled()
    {
        return inPool;
    }
    
    public static Vector2 create(float x, float y)
    {
        Vector2 v = create();
        v.set(x,y);
        return v;
    }
    public static Vector2 create(Vector2 u)
    {
        Vector2 v = create();
        v.set(u);
        return v;
    }
    public static Vector2 create()
    {
        if(1==1)
            return new Vector2();
        
        if(pool[poolPos] == null)
        {
            pool[poolPos] = new Vector2();
            pool[poolPos].inPool = true;
            maxPoolPos = poolPos;
        }
        Vector2 v = pool[poolPos];
        v.set(0,0);

        poolPos++;
        return v;
    }
    
    public static void flushPool()
    {
        poolPos = 0;
    }
    
    public Vector2()
    {
        vecs++;
        inPool = false;
	set(0,0);
    }
    
    public Vector2(float x, float y)
    {
        vecs++;
        inPool = false;
	set(x,y);
    }
    public Vector2(double x, double y)
    {
        vecs++;
        inPool = false;
	set((float)x,(float)y);
    }
    
    public Vector2(Vector2 v)
    {
        vecs++;
        inPool = false;
        set(v.x,v.y);
    }
    
    public Vector3 toVec3()
    {
        return new Vector3(x,y,0);
    }
    
    public float getAngle()
    {
        return (float)Math.toDegrees(Math.atan2(y, x));
    }
    
    public boolean zero()
    {
        return (x == 0 && y == 0);
    }
    
    public void set(float x, float y)
    {
	this.x = x;
	this.y = y;
    }    
    public void setX(float x)
    {
	this.x = x;
    }
    public void setY(float y)
    {
	this.y = y;
    }    
    public void set(Vector2 v)
    {
	this.x = v.x;
	this.y = v.y;
    }
    
    public void move(float xx, float yy)
    {
	x += xx;
	y += yy;
    }
    
    public float getX()
    {
	return x;
    }
    public float getY()
    {
	return y;
    }
    
    public Vector2 add(Vector2 b)
    {
	return Vector2.create(x+b.x, y+b.y);
    }
    
    public Vector2 negate()
    {
	return Vector2.create(-x,-y);
    }
    
    public Vector2 sub(Vector2 b)
    {
	return Vector2.create(x-b.x, y-b.y);
    }
    
    public Vector2 scale(float c)
    {
	return Vector2.create(x*c, y*c);
    }
    
    public float dot(Vector2 b)
    {
	return (x*b.x) + (y*b.y);
    }
    
    public float length()
    {
	return (float)Math.sqrt( (double)(x*x + y*y) );
    }
    
    public Vector2 perp()
    {
	return Vector2.create(-y,x);
    }
    
    public Vector2 norm()
    {
	float mag = length();
	return Vector2.create(x/mag, y/mag);
    }
    
    public float angle(Vector2 b)
    {
        float dx = b.x - x;
        float dy = b.y - y;
        return (float)Math.atan2(dy,dx);
	//return (float)Math.acos( this.dot(b) / (this.length() * b.length()) );
    }
    
    public static Vector2 fromAngle(float deg)
    {
        float rad = (float)Math.toRadians(deg);
        return Vector2.create((float)Math.cos(rad), (float)Math.sin(rad));
    }
    
    public boolean equals(Vector2 b)
    {
        return equals(b,EPSILON);
    }
    public boolean equals(Vector2 b, float ep)
    {
        return Math.abs( x-b.x ) < ep && Math.abs( y-b.y ) < ep;
    }
    
    public String toString()
    {
        return "("+x+","+y+")";
    }
}
