/*
 * Vector3.java
 *
 * Created on September 8, 2007, 2:50 PM
 *
 */
package irrtum.engine;


/**
 * Utility class with the purpose of handling 3-dimensional coordinates
 * with relative ease.
 */
public class Vector3
{
    public static final float EPSILON = 0.00001f;
    public float x, y, z;

    public Vector3()
    {
        set(0,0,0);
    }

    public Vector3(float x, float y, float z)
    {
        set(x,y,z);
    }

    public Vector3(Vector3 v)
    {
        set(v.x,v.y,v.z);
    }

    public Vector3(Vector2 v)
    {
        x = v.x;
        y = v.y;
        z = 0;
    }

    public static Vector3 random()
    {
        return new Vector3((float)Math.random(), (float)Math.random(), (float)Math.random());
    }

    public boolean zero()
    {
        return (x == 0 && y == 0 && z == 0);
    }

    public void set(float x, float y, float z)
    {
        this.x = x;
        this.y = y;
        this.z = z;
    }    
    public void setX(float x)
    {
	this.x = x;
    }
    public void setY(float y)
    {
	this.y = y;
    }
    public void setZ(float z)
    {
        this.z =z;
    }
    public void set(Vector3 v)
    {
        this.x = v.x;
        this.y = v.y;
        this.z = v.z;
    }

    public void move(float xx, float yy, float zz)
    {
        x += xx;
        y += yy;
        z += zz;
    }

    public float getX()
    {
        return x;
    }
    public float getY()
    {
        return y;
    }
    public float getZ()
    {
        return z;
    }

    public Vector3 add(Vector3 b)
    {
        return new Vector3(x+b.x, y+b.y, z+b.z);
    }

    public Vector3 negate()
    {
        return new Vector3(-x,-y,-z);
    }

    public Vector3 sub(Vector3 b)
    {
        return new Vector3(x-b.x, y-b.y, z-b.z);
    }

    public Vector3 scale(float c)
    {
        return new Vector3(x*c, y*c, z*c);
    }

    public float dot(Vector3 b)
    {
        return (x*b.x) + (y*b.y) + (z*b.z);
    }

    public float length()
    {
        return (float)Math.sqrt( (double)(x*x + y*y + z*z) );
    }
        
    public Vector3 norm()
    {
        float mag = length();
        return new Vector3(x/mag, y/mag, z/mag);
    }

    public float angle(Vector3 b)
    {
        return (float)Math.acos( this.dot(b) / (this.length() * b.length()) );
    }

    public boolean equals(Vector3 b)
    {
        return equals(b,EPSILON);
    }
    public boolean equals(Vector3 b, float ep)
    {
        return Math.abs( x-b.x ) < ep && Math.abs( y-b.y ) < ep && Math.abs( z-b.z ) < ep;
    }

    public Vector2 to2D()
    {
        return new Vector2(x,y);
    }

    public Vector3 cross(Vector3 v)
    {
        return new Vector3( y*v.z - z*v.y, z*v.x - x*v.z, x*v.y - y*v.x );
//            (a2b3 − a3b2, a3b1 − a1b3, a1b2 − a2b1).
    }

    public Vector3 rotateY(float deg)
    {
        float theta = (float)Math.toRadians(-deg);
        float newX = (float)(Math.cos(theta) * x + Math.sin(theta) * z);
        float newZ = (float)(Math.cos(theta) * z - Math.sin(theta) * x);
        return new Vector3(newX, y, newZ);
    }

    public String toString()
    {
        return "("+x+","+y+","+z+")";
    }
}
