/*
 * BillboardEntity.java
 *
 * Created: September 30, 2011
 * Author: Stephen Whitmore
 *
 */
package irrtum.game;

import irrtum.engine.AABB;
import irrtum.engine.Texture;
import org.lwjgl.opengl.GL11;


/**
 * In-game entity represented by a flat billboard.
 *
 */
public class BillboardEntity extends Entity
{
    protected Texture texture;
    
    public BillboardEntity()
    {
        super();
        name = "Billboard Entity";
    }
    
    public void render(Camera camera)
    {
        if(texture != null)
        {
            //Billboard.draw(texture, camera, pos, scale, new Vector2(texture.getWidth(),texture.getHeight()), 0, new Vector3(1,1,1));
            GL11.glPushMatrix();
            GL11.glLoadIdentity();
            camera.doTransform();
            GL11.glTranslatef(pos.x, pos.y, pos.z);
            GL11.glScalef(scale,scale,scale);
            float yRot = (float)Math.toDegrees( Math.atan2(camera.pos.x-pos.x, camera.pos.z-pos.z) );
            GL11.glRotatef(yRot, 0,1,0);
            texture.bind();

            // Draw the billboard
            //GL11.glColor3f(col.x, col.y, col.z);
            GL11.glBegin(GL11.GL_QUADS);
                GL11.glTexCoord2f(0,1);
                GL11.glVertex2f(-8,-8);

                GL11.glTexCoord2f(1,1);
                GL11.glVertex2f(8,-8);

                GL11.glTexCoord2f(1,0);
                GL11.glVertex2f(8,8);

                GL11.glTexCoord2f(0,0);
                GL11.glVertex2f(-8,8);
            GL11.glEnd();

            // Clean up changed state information
            GL11.glPopMatrix();
            GL11.glColor4f(1,1,1,1);
        }
    }
    
    public void update()
    {
        super.update();
    }

    public void setTexture(Texture texture)
    {
        this.texture = texture;
    }
    public Texture getTexture()
    {
        return texture;
    }
}
