/*
 * Camera.java
 *
 * Created: April 7, 2009, 12:55 PM
 * Author: Stephen Whitmore
 *
 */
package irrtum.game;

import irrtum.engine.Vector3;
import org.lwjgl.util.glu.*;


/**
 * The abstract 3D viewport that the player uses to look into the game world.
 */
public class Camera
{
    public Vector3 pos, look;
    private Vector3 rot;
    public float viewDistance;
    
    public Camera(float vd)
    {
        pos = new Vector3();
        look = new Vector3();
        rot = new Vector3();
        viewDistance = vd;
    }
       
    public void setPos(Vector3 v)
    {
        pos.set(v);
    }
    
    public void setLook(Vector3 v)
    {
    	look.set(v);
    }

    public void rotateY(float deg)
    {
        rot.y += deg;

        recalculateLook();
    }

    public void rotateX(float deg)
    {
        rot.x += deg;
        rot.x = (float)Math.max(-90, Math.min(90, rot.x));

        recalculateLook();
    }

    private void recalculateLook()
    {
        float x, y, z;

        look.set(0,0,1);

        // X rotation.
        float alpha = (float)Math.toRadians(-rot.x);
        y = (float)(Math.cos(alpha) * look.y - Math.sin(alpha) * look.z);
        z = (float)(Math.cos(alpha) * look.z + Math.sin(alpha) * look.y);
        look.y = y;
        look.z = z;

        // Y rotation.
        float theta = (float)Math.toRadians(-rot.y);
        x = (float)(Math.cos(theta) * look.x + Math.sin(theta) * look.z);
        z = (float)(Math.cos(theta) * look.z - Math.sin(theta) * look.x);
        look.x = x;
        look.z = z;

//        look = look.norm();
    }

    public void doTransform()
    {
        //GL11.glLoadIdentity();
        //GL11.glRotatef(-90, 1, 0, 0);
        //GL11.glRotatef((float)Math.toDegrees(Math.atan2(look.x,look.y)), 0, 0, 1);
        GLU.gluLookAt(pos.x, pos.y, pos.z, pos.x+look.x, pos.y+look.y, pos.z+look.z, 0, 1, 0);
        //GL11.glTranslatef( -pos.x, -pos.y, -pos.z );
        //Vector3 ahead = pos.sub(look);
        //GLU.gluLookAt(pos.x, pos.y, pos.z, ahead.x, ahead.y, ahead.z, 0, 1, 0);
    }
}
