/*
 * Entity.java
 *
 * Created: September 30, 2011
 * Author: Stephen Whitmore
 *
 */
package irrtum.game;

import irrtum.engine.AABB;
import irrtum.engine.Vector3;


/**
 * In-game entity represented by some sort of visual component.
 *
 */
public class Entity
{
    protected String name;
    protected Vector3 lastPos, pos;
    protected float scale;
    protected Vector3 rot;
    protected boolean dead;
    protected AABB hitBox;
    
    public Entity()
    {
        name = "Entity";
        lastPos = new Vector3();
        pos = new Vector3();
        rot = new Vector3();
        scale = 1.0f;
        dead = false;
    }
    
    public String getName()
    {
        return name;
    }
    public void setName(String name)
    {
        this.name = name;
    }

    public void render(Camera camera)
    {
    }
    
    public void update()
    {
        lastPos.set(pos);
    }
    
    public void collision(Entity spr)
    {
    }
    
    public float distanceTo(Entity spr)
    {
        float x1 = pos.x;
        float y1 = pos.y;
        float x2 = spr.pos.x;
        float y2 = spr.pos.y;
        float dx = x2 - x1;
        float dy = y2 - y1;
        return (float)Math.sqrt( dx*dx + dy*dy );
    }
    
    public void kill()
    {
        // ...
    }

    public AABB getHitbox()
    {
        return hitBox;
    }
    
    public void setHitbox(AABB aabb)
    {
        hitBox = aabb;
    }

    public Vector3 getPos()
    {
        return pos;
    }

    public Vector3 getRot()
    {
        return rot;
    }

    public float getScale()
    {
    	return scale;
    }

    public void setPos(Vector3 pos)
    {
        this.pos.set(pos);
    }

    public boolean isDead()
    {
    	return dead;
    }

    public void setDead(boolean dead)
    {
    	this.dead = dead;
    }

    public void setScale(float scale)
    {
        this.scale = scale;
    }
}
